@@ -10,8 +10,7 @@
 #
 from abc import ABCMeta, abstractmethod
 from .compat import add_metaclass
-from .exceptions import ElementPathTypeError, ElementPathValueError
-from .namespaces import XSD_NAMESPACE
+from .exceptions import ElementPathTypeError
 from .xpath_nodes import is_etree_element
 from .xpath_context import XPathSchemaContext
 
@@ -106,10 +105,20 @@ class AbstractXsdType(AbstractXsdComponent):
         `False` otherwise.
         """
 
+    @abstractmethod
+    def validate(self, obj, *args, **kwargs):
+        """
+        Validates an XML object node using the XSD type. The argument *obj* is an element
+        for complex type nodes or a text value for simple type nodes. Raises a `ValueError`
+        compatible exception (a `ValueError` or a subclass of it) if the argument is not valid.
+        """
+
     @abstractmethod
     def decode(self, obj, *args, **kwargs):
         """
-        Decodes XML data using the XSD type.
+        Decodes an XML object node using the XSD type. The argument *obj* is an element
+        for complex type nodes or a text value for simple type nodes. Raises a `ValueError`
+        or a `TypeError` compatible exception if the argument it's not valid.
         """
 
 
@@ -135,6 +144,21 @@ class AbstractSchemaProxy(object):
         self._schema = schema
         self._base_element = base_element
 
+    def bind_parser(self, parser):
+        """
+        Binds a parser instance with schema proxy adding the schema's atomic types constructors.
+        This method can be redefined in a concrete proxy to optimize schema bindings.
+
+        :param parser: a parser instance.
+        """
+        if parser.schema is not self:
+            parser.schema = self
+
+        parser.symbol_table = parser.__class__.symbol_table.copy()
+        for xsd_type in self.iter_atomic_types():
+            parser.schema_constructor(xsd_type.name)
+        parser.tokenizer = parser.create_tokenizer(parser.symbol_table)
+
     def get_context(self):
         """
         Get a context instance for static analysis phase.
@@ -143,6 +167,15 @@ class AbstractSchemaProxy(object):
         """
         return XPathSchemaContext(root=self._schema, item=self._base_element)
 
+    def find(self, path, namespaces=None):
+        """
+        Find a schema element or attribute using an XPath expression.
+
+        :param path: an XPath expression that selects an element or an attribute node.
+        :param namespaces: an optional mapping from namespace prefix to namespace URI.
+        :return: The first matching schema component, or ``None`` if there is no match.
+        """
+
     @abstractmethod
     def get_type(self, qname):
         """
@@ -224,79 +257,5 @@ class AbstractSchemaProxy(object):
         """
 
 
-class XMLSchemaProxy(AbstractSchemaProxy):
-    """
-    Schema proxy for the *xmlschema* library. It will be removed soon because
-    xmlschema v1.0.14 will includes an its own version of schema proxy that
-    uses a custom context implementation that recognizes circular references.
-    """
-    def __init__(self, schema=None, base_element=None):
-        if schema is None:
-            from xmlschema import XMLSchema
-            schema = XMLSchema.meta_schema
-        super(XMLSchemaProxy, self).__init__(schema, base_element)
-
-        if base_element is not None:
-            try:
-                if base_element.schema is not schema:
-                    raise ElementPathValueError("%r is not an element of %r" % (base_element, schema))
-            except AttributeError:
-                raise ElementPathTypeError("%r is not an XsdElement" % base_element)
-
-    def get_type(self, qname):
-        try:
-            return self._schema.maps.types[qname]
-        except KeyError:
-            return None
-
-    def get_attribute(self, qname):
-        try:
-            return self._schema.maps.attributes[qname]
-        except KeyError:
-            return None
-
-    def get_element(self, qname):
-        try:
-            return self._schema.maps.elements[qname]
-        except KeyError:
-            return None
-
-    def get_substitution_group(self, qname):
-        try:
-            return self._schema.maps.substitution_groups[qname]
-        except KeyError:
-            return None
-
-    def is_instance(self, obj, type_qname):
-        xsd_type = self._schema.maps.types[type_qname]
-        try:
-            xsd_type.encode(obj)
-        except ValueError:
-            return False
-        else:
-            return True
-
-    def cast_as(self, obj, type_qname):
-        xsd_type = self._schema.maps.types[type_qname]
-        return xsd_type.decode(obj)
-
-    def iter_atomic_types(self):
-        for xsd_type in self._schema.maps.types.values():
-            if xsd_type.target_namespace != XSD_NAMESPACE and hasattr(xsd_type, 'primitive_type'):
-                yield xsd_type
-
-    def get_primitive_type(self, xsd_type):
-        if not xsd_type.is_simple():
-            return self._schema.maps.types['{%s}anyType' % XSD_NAMESPACE]
-        elif not hasattr(xsd_type, 'primitive_type'):
-            if xsd_type.base_type is None:
-                return xsd_type
-            return self.get_primitive_type(xsd_type.base_type)
-        elif xsd_type.primitive_type is not xsd_type:
-            return self.get_primitive_type(xsd_type.primitive_type)
-        else:
-            return xsd_type
-
-
-__all__ = ['AbstractXsdComponent', 'AbstractEtreeElement', 'AbstractXsdType', 'AbstractXsdAttribute',
-           'AbstractXsdElement', 'AbstractSchemaProxy', 'XMLSchemaProxy']
+__all__ = ['AbstractXsdComponent', 'AbstractEtreeElement', 'AbstractXsdType',
+           'AbstractXsdAttribute', 'AbstractXsdElement', 'AbstractSchemaProxy']
