@@ -38,16 +38,23 @@ constructor = XPath2Parser.constructor
 # Constructors for string-based XSD types
 @constructor('normalizedString')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
     return str(value).replace('\t', ' ').replace('\n', ' ')
 
 
 @constructor('token')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
     return collapse_white_spaces(value)
 
 
 @constructor('language')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
+
     match = LANGUAGE_CODE_PATTERN.match(collapse_white_spaces(value))
     if match is None:
         raise xpath_error('FOCA0002', "%r is not a language code" % value)
@@ -56,6 +63,9 @@ def cast(value):
 
 @constructor('NMTOKEN')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
+
     match = NMTOKEN_PATTERN.match(collapse_white_spaces(value))
     if match is None:
         raise xpath_error('FOCA0002', "%r is not an xs:NMTOKEN value" % value)
@@ -64,6 +74,9 @@ def cast(value):
 
 @constructor('Name')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
+
     match = NAME_PATTERN.match(collapse_white_spaces(value))
     if match is None:
         raise xpath_error('FOCA0002', "%r is not an xs:Name value" % value)
@@ -75,6 +88,9 @@ def cast(value):
 @constructor('IDREF')
 @constructor('ENTITY')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
+
     match = NCNAME_PATTERN.match(collapse_white_spaces(value))
     if match is None:
         raise xpath_error('FOCA0002', "invalid value %r for constructor" % value)
@@ -83,6 +99,9 @@ def cast(value):
 
 @constructor('anyURI')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
+
     uri = collapse_white_spaces(value)
     try:
         urlparse(uri)
@@ -99,6 +118,8 @@ def cast(value):
 # Constructors for numeric XSD types
 @constructor('decimal')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
     try:
         return decimal.Decimal(value)
     except (ValueError, decimal.DecimalException) as err:
@@ -108,6 +129,8 @@ def cast(value):
 @constructor('double')
 @constructor('float')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
     try:
         return float(value)
     except ValueError as err:
@@ -125,6 +148,9 @@ def cast_to_integer(value, lower_bound=None, higher_bound=None):
     :raise: an `ElementPathValueError` if the value is not decodable to an integer or if \
     the value is out of bounds.
     """
+    if isinstance(value, tuple):
+        value = value[-1]
+
     if isinstance(value, string_base_type):
         try:
             result = int(float(value))
@@ -212,6 +238,8 @@ def cast(value):
 # Constructors for datetime XSD types
 @constructor('date')
 def cast(value, tz=None):
+    if isinstance(value, tuple):
+        value = value[-1]
     if isinstance(value, Date10):
         return value
     return Date10.fromstring(value, tzinfo=tz)
@@ -219,6 +247,8 @@ def cast(value, tz=None):
 
 @constructor('gDay')
 def cast(value, tz=None):
+    if isinstance(value, tuple):
+        value = value[-1]
     if isinstance(value, XPathGregorianDay):
         return value
     return XPathGregorianDay.fromstring(value, tzinfo=tz)
@@ -226,6 +256,8 @@ def cast(value, tz=None):
 
 @constructor('gMonth')
 def cast(value, tz=None):
+    if isinstance(value, tuple):
+        value = value[-1]
     if isinstance(value, XPathGregorianMonth):
         return value
     return XPathGregorianMonth.fromstring(value, tzinfo=tz)
@@ -233,6 +265,8 @@ def cast(value, tz=None):
 
 @constructor('gMonthDay')
 def cast(value, tz=None):
+    if isinstance(value, tuple):
+        value = value[-1]
     if isinstance(value, XPathGregorianMonthDay):
         return value
     return XPathGregorianMonthDay.fromstring(value, tzinfo=tz)
@@ -240,6 +274,8 @@ def cast(value, tz=None):
 
 @constructor('gYear')
 def cast(value, tz=None):
+    if isinstance(value, tuple):
+        value = value[-1]
     if isinstance(value, XPathGregorianYear):
         return value
     return XPathGregorianYear.fromstring(value, tzinfo=tz)
@@ -247,6 +283,8 @@ def cast(value, tz=None):
 
 @constructor('gYearMonth')
 def cast(value, tz=None):
+    if isinstance(value, tuple):
+        value = value[-1]
     if isinstance(value, XPathGregorianYearMonth):
         return value
     return XPathGregorianYearMonth.fromstring(value, tzinfo=tz)
@@ -254,6 +292,8 @@ def cast(value, tz=None):
 
 @constructor('time')
 def cast(value, tz=None):
+    if isinstance(value, tuple):
+        value = value[-1]
     if isinstance(value, Time):
         return value
     return Time.fromstring(value, tzinfo=tz)
@@ -282,16 +322,28 @@ def evaluate(self, context=None):
 # Constructors for time durations XSD types
 @constructor('duration')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
+    if isinstance(value, Duration):
+        return value
     return Duration.fromstring(value)
 
 
 @constructor('yearMonthDuration')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
+    if isinstance(value, YearMonthDuration):
+        return value
     return YearMonthDuration.fromstring(value)
 
 
 @constructor('dayTimeDuration')
 def cast(value):
+    if isinstance(value, tuple):
+        value = value[-1]
+    if isinstance(value, DayTimeDuration):
+        return value
     return DayTimeDuration.fromstring(value)
 
 
@@ -299,6 +351,8 @@ def cast(value):
 # Constructors for binary XSD types
 @constructor('base64Binary')
 def cast(value, from_literal=False):
+    if isinstance(value, tuple):
+        value = value[-1]
     if isinstance(value, UntypedAtomic):
         return codecs.encode(unicode_type(value), 'base64')
     elif not isinstance(value, (bytes, unicode_type)):
@@ -316,6 +370,8 @@ def cast(value, from_literal=False):
 
 @constructor('hexBinary')
 def cast(value, from_literal=False):
+    if isinstance(value, tuple):
+        value = value[-1]
     if isinstance(value, UntypedAtomic):
         return codecs.encode(unicode_type(value), 'hex')
     elif not isinstance(value, (bytes, unicode_type)):
@@ -398,9 +454,9 @@ def select(self, context=None):
             for result in self[0].select(context):
                 yield result
     else:
-        attribute_name = self[0].evaluate(context) if self else None
+        name = self[0].evaluate(context) if self else None
         for item in context.iter_attributes():
-            if is_attribute_node(item, attribute_name):
+            if is_attribute_node(item, name):
                 yield context.item[1]
 
 
@@ -419,7 +475,7 @@ def cast_to_boolean(value, context=None):
     elif isinstance(value, (int, float, decimal.Decimal)):
         return bool(value)
     elif isinstance(value, UntypedAtomic):
-        value = string_base_type(value)
+        value = unicode_type(value)
     elif not isinstance(value, string_base_type):
         raise xpath_error('FORG0006', 'the argument has an invalid type %r' % type(value))
 
@@ -599,4 +655,4 @@ def evaluate(self, context=None):
         return DateTime10(dt.year, dt.month, dt.day, tm.hour, tm.minute, tm.second, tm.microsecond, tzinfo)
 
 
-XPath2Parser.build_tokenizer()  # XPath 2.0 definitions completed, build the tokenizer.
+XPath2Parser.build()  # XPath 2.0 definition complete, can build the parser class.
