@@ -18,7 +18,7 @@ import decimal
 import math
 import operator
 
-from .compat import MutableSequence, urlparse
+from .compat import MutableSequence, urlparse, unicode_type
 from .exceptions import ElementPathError, ElementPathKeyError, \
     ElementPathTypeError, MissingContextError
 from .namespaces import XSD_NAMESPACE, XPATH_FUNCTIONS_NAMESPACE, \
@@ -26,8 +26,7 @@ from .namespaces import XSD_NAMESPACE, XPATH_FUNCTIONS_NAMESPACE, \
     qname_to_prefixed, prefixed_to_qname, XSD_UNTYPED_ATOMIC
 from .datatypes import UntypedAtomic, XSD_BUILTIN_TYPES
 from .xpath_nodes import is_xpath_node
-from .tdop_parser import create_tokenizer
-from .xpath1_parser import XML_NCNAME_PATTERN, XPath1Parser
+from .xpath1_parser import XPath1Parser
 from .xpath_context import XPathSchemaContext
 from .schema_proxy import AbstractSchemaProxy
 
@@ -167,19 +166,21 @@ class XPath2Parser(XPath1Parser):
             self.function_namespace = function_namespace
 
         if schema is None:
-            self.schema = None
+            pass
         elif not isinstance(schema, AbstractSchemaProxy):
-            raise ElementPathTypeError("schema argument must be a subclass or instance of AbstractSchemaProxy!")
+            raise ElementPathTypeError("argument 'schema' must be an instance of AbstractSchemaProxy")
         else:
-            self.schema = schema
-            self.symbol_table = self.symbol_table.copy()
-            for xsd_type in self.schema.iter_atomic_types():
-                self.schema_constructor(xsd_type.name)
-            self.tokenizer = create_tokenizer(self.symbol_table, XML_NCNAME_PATTERN)
+            schema.bind_parser(self)
 
         self.base_uri = None if base_uri is None else urlparse(base_uri).geturl()
         self._compatibility_mode = compatibility_mode
 
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        state.pop('symbol_table', None)
+        state.pop('tokenizer', None)
+        return state
+
     @property
     def version(self):
         return '2.0'
@@ -337,9 +338,11 @@ class XPath2Parser(XPath1Parser):
         except KeyError:
             raise ElementPathKeyError("unknown type %r" % type_qname)
 
+    def is_schema_bound(self):
+        return 'symbol_table' in self.__dict__
+
     def parse(self, source):
         root_token = super(XPath1Parser, self).parse(source)
-
         if self.schema is None:
             try:
                 root_token.evaluate()  # Static context evaluation
@@ -381,34 +384,25 @@ register('?')
 register('(:')
 register(':)')
 
-
 ###
 # Node sequence composition
-@method(infix('union', bp=50))
-def select(self, context=None):
-    if context is not None:
-        results = {item for k in range(2) for item in self[k].select(context.copy())}
-        for item in context.iter():
-            if item in results:
-                yield item
+XPath2Parser.duplicate('|', 'union')
 
 
 @method(infix('intersect', bp=55))
 def select(self, context=None):
     if context is not None:
         results = set(self[0].select(context.copy())) & set(self[1].select(context.copy()))
-        for item in context.iter():
-            if item in results:
-                yield item
+        for item in context.iter_results(results):
+            yield item
 
 
 @method(infix('except', bp=55))
 def select(self, context=None):
     if context is not None:
         results = set(self[0].select(context.copy())) - set(self[1].select(context.copy()))
-        for item in context.iter():
-            if item in results:
-                yield item
+        for item in context.iter_results(results):
+            yield item
 
 
 ###
@@ -435,7 +429,7 @@ def evaluate(self, context=None):
 
 @method('if')
 def select(self, context=None):
-    if self.boolean_value(list(self[0].select(context))):
+    if self.boolean_value([x for x in self[0].select(context)]):
         for result in self[1].select(context):
             yield result
     else:
@@ -476,7 +470,8 @@ def evaluate(self, context=None):
     for results in product(*selectors):
         for i in range(len(results)):
             context.variables[self[i * 2][0].value] = results[i]
-        if self.boolean_value(list(self[-1].select(context.copy()))):
+
+        if self.boolean_value([x for x in self[-1].select(context.copy())]):
             if some:
                 return True
         elif not some:
@@ -639,9 +634,10 @@ def evaluate(self, context=None):
         else:
             self.wrong_context_type("an atomic value is required")
 
+    input_value = self.data_value(result[0])
     try:
         if namespace != XSD_NAMESPACE:
-            value = self.parser.schema.cast_as(result[0], atomic_type)
+            value = self.parser.schema.cast_as(input_value, atomic_type)
         else:
             local_name = atomic_type.split('}')[1]
             token_class = self.parser.symbol_table.get(local_name)
@@ -649,13 +645,13 @@ def evaluate(self, context=None):
                 self.unknown_atomic_type("atomic type %r not found in the in-scope schema types" % self[1].source)
 
             if local_name in {'base64Binary', 'hexBinary'}:
-                value = token_class.cast(result[0], self[0].label == 'literal')
+                value = token_class.cast(input_value, self[0].label == 'literal')
             elif local_name in {'dateTime', 'date', 'gDay', 'gMonth', 'gMonthDay', 'gYear', 'gYearMonth', 'time'}:
-                value = token_class.cast(result[0], tz=None if context is None else context.timezone)
+                value = token_class.cast(input_value, tz=None if context is None else context.timezone)
             elif local_name == 'QName':
-                value = token_class.cast(result[0], self.parser.namespaces)
+                value = token_class.cast(input_value, self.parser.namespaces)
             else:
-                value = token_class.cast(result[0])
+                value = token_class.cast(input_value)
 
     except ElementPathError as err:
         if self.symbol != 'cast':
@@ -668,7 +664,7 @@ def evaluate(self, context=None):
     except TypeError as err:
         if self.symbol != 'cast':
             return False
-        self.wrong_type(str(err))
+        self.wrong_type(unicode_type(err))
     except ValueError as err:
         if self.symbol != 'cast':
             return False
@@ -753,13 +749,13 @@ def evaluate(self, context=None):
 def evaluate(self, context=None):
     symbol = self.symbol
 
-    left = list(self[0].select(context))
+    left = [x for x in self[0].select(context)]
     if not left:
         return
     elif len(left) > 1 or not is_xpath_node(left[0]):
         self[0].wrong_type("left operand of %r must be a single node" % symbol)
 
-    right = list(self[1].select(context))
+    right = [x for x in self[1].select(context)]
     if not right:
         return
     elif len(right) > 1 or not is_xpath_node(right[0]):
@@ -791,7 +787,7 @@ def evaluate(self, context=None):
             self.wrong_type(str(err))
         return
     else:
-        return list(range(start, stop))
+        return [x for x in range(start, stop)]
 
 
 @method('to')
